<?php

namespace Ranig;

use ManiaControl\ManiaControl;
use ManiaControl\Callbacks\CallbackListener;
use ManiaControl\Callbacks\Callbacks;
use ManiaControl\Callbacks\Structures\TrackMania\OnScoresStructure;
use ManiaControl\Commands\CommandListener;
use ManiaControl\Plugins\Plugin;
use ManiaControl\Maps\Map;
use ManiaControl\Maps\MapManager;
use ManiaControl\Maps\MapQueue;
use ManiaControl\Players\Player;
use ManiaControl\Players\PlayerManager;
use MCTeam\LocalRecordsPlugin;

/**
 */
class AutoQueuePlugin implements CallbackListener, CommandListener, Plugin {
	/*
	 * Constants
	 */
	const PLUGIN_ID                     = 173;
	const PLUGIN_VERSION                = 1;
	const PLUGIN_NAME                   = 'Auto Queue Plugin';
	const PLUGIN_AUTHOR                 = 'Ranig';
	const DEFAULT_LOCAL_RECORDS_PLUGIN  = 'MCTeam\LocalRecordsPlugin';
	const SETTING_AUTOQUEUE             = 'define if autoqueue is ON or OFF';
	const SETTING_SUSPEND_COUNT         = 'number of maps during which the autoqueue is suspended';

	/*
	 * Private properties
	 */
	/** @var ManiaControl $maniaControl * */
	private $maniaControl = null;

	/** @var LocalRecordsPlugin $records * */
	private $records = null;

	/** @var MapManager $manager * */
	private $manager = null;

	/** @var PlayerManager $manager * */
	private $players = null;
	
	/** @var int $suspend */
	private $suspend = 0;
	
	/** @var bool $locked */
	private $locked = false;
	
	
	/**
	 * @see \ManiaControl\Plugins\Plugin::prepare()
	 */
	public static function prepare(ManiaControl $maniaControl) {
	}

	/**
	 * @see \ManiaControl\Plugins\Plugin::getId()
	 */
	public static function getId() {
		return self::PLUGIN_ID;
	}

	/**
	 * @see \ManiaControl\Plugins\Plugin::getName()
	 */
	public static function getName() {
		return self::PLUGIN_NAME;
	}

	/**
	 * @see \ManiaControl\Plugins\Plugin::getVersion()
	 */
	public static function getVersion() {
		return self::PLUGIN_VERSION;
	}

	/**
	 * @see \ManiaControl\Plugins\Plugin::getAuthor()
	 */
	public static function getAuthor() {
		return self::PLUGIN_AUTHOR;
	}

	/**
	 * @see \ManiaControl\Plugins\Plugin::getDescription()
	 */
	public static function getDescription() {
		return "Auto Queue Plugin, autoqueue maps";
	}

	/**
	 * @see \ManiaControl\Plugins\Plugin::load()
	 */
	public function load(ManiaControl $maniaControl) {
		$this->maniaControl = $maniaControl;

		$this->records = $this->maniaControl->getPluginManager()->getPlugin(self::DEFAULT_LOCAL_RECORDS_PLUGIN);
		$this->manager = $this->maniaControl->getMapManager();
		$this->players = $this->maniaControl->getPlayerManager();
		
		if (!$this->records) {
			$this->maniaControl->getChat()->sendError('[AutoQueuePlugin] You need to activate ' . self::DEFAULT_LOCAL_RECORDS_PLUGIN);
		}

		$this->maniaControl->getCommandManager()->registerCommandListener('autojuke', $this, 'command_autojuke', false, 'Autojukes a map with no local record.');
		$this->maniaControl->getCommandManager()->registerCommandListener('autoqueue', $this, 'command_autoqueue', false, 'Displays the status of the autoqueue.');
		$this->maniaControl->getCommandManager()->registerCommandListener('stopautoqueue', $this, 'command_suspend', false, 'Suspends the autoqueue for a while.');
		$this->maniaControl->getCommandManager()->registerCommandListener('resumeautoqueue', $this, 'command_resume', true, 'Resumes the autoqueue if it has been suspended.');
		$this->maniaControl->getCallbackManager()->registerCallbackListener(Callbacks::TM_SCORES, $this, 'handleScores');

		$this->maniaControl->getSettingManager()->initSetting($this, self::SETTING_AUTOQUEUE, false);
		$this->maniaControl->getSettingManager()->initSetting($this, self::SETTING_SUSPEND_COUNT, 10);
	}


	/**
	 * @see \ManiaControl\Plugins\Plugin::unload()
	 */
	public function unload() {
	}
	
	/**
	 * Returns one of the maps with the worst records for the current player
	 *
	 * @param Player $player
	 */
	private function findMapForPlayer(Player $player) {
		$maps = $this->manager->getMaps();
		$buffer = $this->manager->getMapQueue()->getQueueBuffer();
		
		$good = array();
		$maxrank = 1;
		foreach ($maps as $map) {
			
			if (in_array($map->uid, $buffer)) {
				continue;
			}
			
			$rec = $this->records->getLocalRecord($map, $player);
			if ($rec) {
				if ($maxrank < 0) {
					continue;
				}
				
				$rank = $rec->rank;
				if ($rank < $maxrank) {
					continue;
				}
				
				if ($rank > $maxrank) {
					$good = array();
					$maxrank = $rank;
				}
				
				array_push($good, $map);
			} else {
				if ($maxrank >= 0) {
					$good = array();
					$maxrank = -1;
				}
				array_push($good, $map);
			}
		}
		
		if (count($good) == 0) {
			$this->maniaControl->getChat()->sendInformation("no maps available for autojuke", $player);
			return null;
		}

		return $good[array_rand($good)];
	}

	/**
	 * Handles /autojuke command, autojukes a random map
	 *
	 * @param array  $chatCallback
	 * @param Player $player
	 */
	public function command_autojuke(array $chatCallback, Player $player) {
		$map = $this->findMapForPlayer($player);
		if ($map) {
			$this->manager->getMapQueue()->addMapToMapQueue($player->login, $map->uid);
		}
	}

	/**
	 * Handles /autoqueue command, displays the status of the autoqueue in the chat
	 *
	 * @param array  $chatCallback
	 * @param Player $player
	 */
	public function command_autoqueue(array $chatCallback, Player $player) {
		$message = "autoqueue is ";
		if ($this->maniaControl->getSettingManager()->getSettingValue($this, self::SETTING_AUTOQUEUE)) {
			if ($this->suspend > 0) {
				$message = $message . "suspended for " . $this->suspend . " map(s)";
			} else {
				$message = $message . "ON";
			}
		} else {
			$message = $message . "OFF";
		}
		$this->maniaControl->getChat()->sendInformation($message, $player);		
	}

	/**
	 * Handles /stopautoqueue command, temporary suspends autoqueue for 10 maps
	 *
	 * @param array  $chatCallback
	 * @param Player $player
	 */
	public function command_suspend(array $chatCallback, Player $player) {
		if ($this->maniaControl->getSettingManager()->getSettingValue($this, self::SETTING_AUTOQUEUE)) {
			if ($this->locked) {
				$this->maniaControl->getChat()->sendInformation("autoqueue is locked and can't be suspended", $player);
				return;
			}
			
			$this->suspend = $this->maniaControl->getSettingManager()->getSettingValue($this, self::SETTING_SUSPEND_COUNT);
			$this->maniaControl->getChat()->sendInformation("autoqueue has been suspended by " . $player->nickname);
		}
	}

	/**
	 * Handles resumeautoqueue admin command, resumes the autoqueue if it has been suspended
	 *
	 * @param array  $chatCallback
	 * @param Player $player
	 */
	public function command_resume(array $chatCallback, Player $player) {
		if ($this->maniaControl->getSettingManager()->getSettingValue($this, self::SETTING_AUTOQUEUE)) {
			$this->suspend = 0;
			$this->maniaControl->getChat()->sendInformation("autoqueue has been resumed by " . $player->nickname);
			$this->locked = true;
		}
	}
		
	/**
	 * Handles the autoqueue when scores are presented on screen
	 *
	 * @param OnScoresStructure  $structure
	 */
	public function handleScores(OnScoresStructure $fullScoreStructure) {
		if ($fullScoreStructure->getSection() != 'EndMap') {
			return;
		}
		
		if ($this->locked) {
			$this->locked = false;
		}
		
		if (!($this->maniaControl->getSettingManager()->getSettingValue($this, self::SETTING_AUTOQUEUE))) {
			return;
		}
		
		if ($this->suspend > 0) {
			$this->suspend = $this->suspend - 1;
			return;
		}
		
		if (count($this->manager->getMapQueue()->getQueuedMapsRanking()) > 0) {
			return;
		}
		
		$active = $this->players->getPlayers(true);
		shuffle($active);

		foreach ($active as $player) {
			$map = $this->findMapForPlayer($player);
			
			if ($map) {
				$this->manager->getMapQueue()->addMapToMapQueue($player->login, $map);
				return;
			}			
		}
	}
}

